Package: Rlgt
Type: Package
Title: Bayesian Exponential Smoothing Models with Trend Modifications
Version: 0.2-3
URL: https://github.com/cbergmeir/Rlgt
Date: 2025-04-17
Authors@R: c(
    person("Slawek", "Smyl", email = "slaweks@hotmail.co.uk", role = "aut"),
    person("Christoph", "Bergmeir", email = "christoph.bergmeir@monash.edu", role = c("aut", "cre")),
    person("Erwin", "Wibowo", email = "rwinwibowo@gmail.com", role = "aut"),
    person("To Wang", "Ng", email = "edwinnglabs@gmail.com", role = "aut"),
    person("Xueying", "Long", email = "xueying.long@monash.edu", role = "aut"),
    person("Alexander", "Dokumentov", email = "alexander.dokumentov@gmail.com", role = "aut"),
    person("Daniel", "Schmidt", email = "daniel.schmidt@monash.edu", role = "aut"),
    person("Trustees of Columbia University", "", role = c("cph"), comment= "tools/make_cpp.R, R/stanmodels.R"))
Description: An implementation of a number of Global Trend models for time series forecasting 
    that are Bayesian generalizations and extensions of some Exponential Smoothing models. 
    The main differences/additions include 1) nonlinear global trend, 2) Student-t error 
    distribution, and 3) a function for the error size, so heteroscedasticity. The methods 
    are particularly useful for short time series. When tested on the well-known M3 dataset,
    they are able to outperform all classical time series algorithms. The models are fitted 
    with MCMC using the 'rstan' package.
License: GPL-3
Encoding: UTF-8
LazyData: true
ByteCompile: true
Depends: R (>= 3.4.0), Rcpp (>= 0.12.0), methods, rstantools, forecast,
        truncnorm
Imports: rstan (>= 2.26.0), sn
LinkingTo: StanHeaders (>= 2.26.0), rstan (>= 2.26.0), BH (>= 1.66.0),
        Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0), RcppParallel (>=
        5.0.2)
SystemRequirements: GNU make
NeedsCompilation: yes
RoxygenNote: 7.2.3
Suggests: doParallel, foreach, knitr, rmarkdown, Mcomp, RODBC, dplyr,
        ggplot2
VignetteBuilder: knitr
Packaged: 2025-04-19 20:41:23 UTC; xlon0007
Author: Slawek Smyl [aut],
  Christoph Bergmeir [aut, cre],
  Erwin Wibowo [aut],
  To Wang Ng [aut],
  Xueying Long [aut],
  Alexander Dokumentov [aut],
  Daniel Schmidt [aut],
  Trustees of Columbia University [cph] (tools/make_cpp.R,
    R/stanmodels.R)
Maintainer: Christoph Bergmeir <christoph.bergmeir@monash.edu>
Repository: CRAN
Date/Publication: 2025-04-30 11:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 21:09:05 UTC; unix
Archs: Rlgt.so.dSYM
