# Generated by roxygen2: do not edit by hand

S3method(forecast,rlgtfit)
S3method(posterior_interval,rlgtfit)
S3method(print,rlgtfit)
export(blgt.multi.forecast)
export(initModel)
export(rlgt)
export(rlgt.control)
import(Rcpp)
import(methods)
import(rstantools)
importFrom(forecast,forecast)
importFrom(forecast,msts)
importFrom(rstan,extract)
importFrom(rstan,get_sampler_params)
importFrom(rstan,rstan_options)
importFrom(rstan,sampling)
importFrom(rstan,stan_model)
importFrom(rstantools,posterior_interval)
importFrom(sn,rst)
importFrom(stats,binomial)
importFrom(stats,frequency)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,ts)
importFrom(stats,tsp)
importFrom(truncnorm,rtruncnorm)
importMethodsFrom(rstan,summary)
useDynLib(Rlgt, .registration=TRUE)
